((app) => {
        function MosaicPanelWidget() {
            const defaultOptions = {
                getTagGradientColor: function (tag) {
                    let isOpera = !!window.opera || navigator.userAgent.indexOf(' OPR/') >= 0;  // Opera 8.0+ (UA detection to detect Blink/v8-powered Opera)
                    let isFirefox = typeof InstallTrigger !== 'undefined';   // Firefox 1.0+
                    let isSafari = Object.prototype.toString.call(window.HTMLElement).indexOf('Constructor') > 0;   // At least Safari 3+: "[object HTMLElementConstructor]"
                    let isChrome = !!window.chrome && !isOpera;              // Chrome 1+
                    // var isIE = /*@cc_on!@*/false || !!document.documentMode; // At least IE6

                    if (isOpera) {
                        return "-o-linear-gradient(right bottom,  rgba(20,46,51,1) 0%, " + tag.color + " 100%)";
                    }
                    if (isFirefox) {
                        return "-moz-linear-gradient(right bottom,  rgba(20,46,51,1) 0%, " + tag.color + " 100%)";
                    }
                    if ((isSafari) || (isChrome)) {
                        return "-webkit-linear-gradient(right bottom,  rgba(20,46,51,1) 0%, " + tag.color + " 100%)";
                    }
                },
                getPreviewPath: function (tagOrDoc) {
                    // eslint-disable-next-line no-undef
                    return Utils.getPreviewPath(tagOrDoc);
                },
                htmlElement: function () {
                    return $('#mosaic_tab')
                },
                isOpen: false,
                pubsub: function () {
                    return app.rekall.Rekall('pubSub');
                },
                rekall: function () {
                    return app.rekall.Rekall('rekall');
                },
                filters_type_to_hide: [],
                filter_label_to_show: [],
                _followedItems: []
            };

            const localOptions = $.extend({}, defaultOptions);

            // eslint-disable-next-line no-unused-vars
            localOptions._resetFilters = function (event) {
                $('.mosaic_filter_item')
                    .removeClass('mosaic_filter_disabled');
                $('.mosaic_label_filter_enabled').click();
                localOptions.filters_type_to_hide = [];
                localOptions.filter_label_to_show = [];
                $('.mosaic_category').show();
            }

            localOptions._destroyFollowedElements = function () {
                let elementsToDestroy = localOptions._followedItems;
                localOptions._followedItems = [];
                elementsToDestroy.forEach(e => e.remove());
            }

            function toggleFilterType(filterName) {
                let filters = localOptions.filters_type_to_hide;
                if (filters.includes(filterName)) {
                    filters.splice(filters.indexOf(filterName), 1);
                } else {
                    filters.push(filterName);
                }
            }

            function toggleLabelToShow(filterName) {
                let filters = localOptions.filter_label_to_show;
                if (filters.includes(filterName)) {
                    filters.splice(filters.indexOf(filterName), 1);
                } else {
                    filters.push(filterName);
                }
            }

            const getUrl = function (tagOrDoc, path) {
                if ('undefined' === typeof path) {
                    let [reg, type] = tagOrDoc.getMetadata("Rekall->Type").split('/');
                    if (reg === 'rekall') {
                        if (type === 'marker') type = 'note';
                    }
                    return tagOrDoc.getDownloadLink() || "../shared/css/images/img-" + type + ".png";
                } else {
                    return path;
                }
            }

            const setDivVisibilityForTag = function (div, tag) {
                if (!localOptions.filter_label_to_show.length) {
                    div.show()
                    return;
                }

                if (tag.getLabels().filter(value => localOptions.filter_label_to_show.includes(value.toLowerCase())).length) {
                    div.show();
                } else {
                    div.hide()
                }
            }

            const createHtmlElementForTagOrDoc = function (tagOrDoc) {
                let path = localOptions.getPreviewPath(tagOrDoc);
                let name = tagOrDoc.getMetadata("Rekall->Name");
                let url = getUrl(tagOrDoc, path);

                let div = $('<div/>').addClass('mosaic_item mosaic_button').on('click', function () {
                    tagOrDoc.openPopupEdit();
                });
                let divImageContainer = $('<div>').addClass('mosaic_item_thumbnail_container');
                divImageContainer
                    .append($('<img src="'+url+'" alt="thumbnail image"/>')
                        .addClass('mosaic_item_thumbnail_container_img')
                        .attr('onerror', "this.src='../shared/css/images/img-document.png';"));
                div.append(divImageContainer);
                div.append($('<span/>')
                    .addClass('caption')
                    .text(name));

                let divUpdater = {
                    div: div,
                    tag: tagOrDoc
                };
                divUpdater.update = function (tag) {
                    setDivVisibilityForTag(div, tag);
                }
                divUpdater.remove = function () {
                    tagOrDoc.removeObserver(divUpdater);
                }
                divUpdater.computeVisibility = function () {
                    setDivVisibilityForTag(divUpdater.div, divUpdater.tag);
                }
                tagOrDoc.addObserver(divUpdater);
                localOptions._followedItems.push(divUpdater);

                setDivVisibilityForTag(div, tagOrDoc);

                return div;
            }

            const getFilterElement = function (text, color, callback, css_class, isToggled) {
                let result = $('<div/>')
                    .addClass('mosaic_filter_item mosaic_button')
                    .addClass(css_class)
                    .on('click', callback)
                    .css('background-color', color)
                    .append($('<h2/>').text(text));


                if (isToggled) {
                    result.toggle();
                    result.toggleClass('mosaic_filter_disabled');
                }

                return result;
            }

            const emptyPanel = function () {
                localOptions.htmlElement().html('');
                localOptions._destroyFollowedElements();
            }

            const fillPanel = function () {
                let resetfilterdiv = $('<div/>').addClass('mosaic_filter');
                let typefilterdiv = $('<div/>').addClass('mosaic_filter');
                let labelsfilterdiv = $('<div/>').addClass('mosaic_filter');
                resetfilterdiv.append(getFilterElement(
                    'Reset filters',
                    'rgb(100,100,100)',
                    localOptions._resetFilters,
                    'mosaic_filter_item_all',
                    false
                ));
                localOptions.htmlElement().append(resetfilterdiv);
                localOptions.htmlElement().append(typefilterdiv);
                localOptions.htmlElement().append(labelsfilterdiv);
                // eslint-disable-next-line no-undef
                let labels = new Set();
                for (let [k, v] of Object.entries(localOptions.rekall().sortings.colors.categories)) {
                    let categoryName = localOptions.rekall().sortings.colors.getCategoryName(k)
                    let category = $('<div/>').addClass('mosaic_category').css('background', localOptions.getTagGradientColor(v));

                    let grid = $('<div/>').addClass('mosaic_category_grid');

                    category.append($('<h2/>').text(categoryName));
                    category.append(grid);

                    typefilterdiv.append(getFilterElement(
                        categoryName,
                        v.color, function () {
                            category.toggle();
                            $(this).toggleClass('mosaic_filter_disabled');
                            localOptions.toggle_filterType(categoryName)
                        },
                        '',
                        localOptions.filters_type_to_hide.includes(categoryName)
                    ));

                    for (let i in v.tags) {
                        grid.append(createHtmlElementForTagOrDoc(v.tags[i]));
                        v.tags[i].getLabels().forEach(l => labels.add(l));
                    }
                    localOptions.htmlElement().append(category);
                }

                Array.from(labels)
                    .sort(function (a, b) {
                        return a.toLowerCase().localeCompare(b.toLowerCase());
                    })
                    .forEach(l => {
                        let button = $('<div/>').html(l);
                        button.addClass('mosaic_filter_item mosaic_button mosaic_label_filter');
                        if (
                            localOptions.filter_label_to_show.length
                            && localOptions.filter_label_to_show.includes(l.toLowerCase())
                        ) {
                            button.toggleClass('mosaic_label_filter_enabled');
                        }

                        button.click(ev => {
                            toggleLabelToShow(l.toLowerCase());
                            $(ev.target).toggleClass('mosaic_label_filter_enabled');
                            ev.target.my_state = !ev.target.my_state;
                            console.debug('filters: ', localOptions.filter_label_to_show);
                            localOptions._followedItems.forEach(value => value.computeVisibility())
                        })
                        labelsfilterdiv.append(button);
                    });
            }

            localOptions.toggle_filterType = toggleFilterType;

            return {
                show: function () {
                    if (localOptions.isOpen)
                        return;

                    localOptions.rekall().timeline.pause();
                    fillPanel();
                    localOptions.htmlElement().show();
                    localOptions.updateObserver = localOptions.pubsub().subscribe('popupEdit.Closed', this.refresh);
                    localOptions.isOpen = true;
                },
                hide: function () {
                    if (!localOptions.isOpen)
                        return;

                    localOptions.pubsub().unsubscribe(localOptions.updateObserver);
                    localOptions.updateObserver = undefined;
                    localOptions.htmlElement().hide();
                    emptyPanel();
                    localOptions.isOpen = false;
                    localOptions._destroyFollowedElements();
                    localOptions.filters_type_to_hide = [];
                },
                refresh: function () {
                    emptyPanel();
                    fillPanel();
                }
            };

        }

        app.rekall.Rekall('mosaicWidget', MosaicPanelWidget());

    }
)(window.app = window.app || {})
